/*
Gnome-shell extension for Liled
Copyright (C) 2012  Yannick Poirier

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
 */


const PanelMenu=imports.ui.panelMenu;
const St=imports.gi.St;
const Main=imports.ui.main;
const Shell=imports.gi.Shell;
const Mainloop=imports.mainloop;
const Lang=imports.lang;
const PopupMenu=imports.ui.popupMenu;
const Util = imports.misc.util;
const Gio = imports.gi.Gio;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Liled = Me.imports.liled;

let mainPanel = null;
let settings = null;

function reportFailure(exception) {
	Main.notifyError("Liled!", exception.message);
}

const MainPanel = new Lang.Class({
	Name: 'Liled.Main',
	Extends: PanelMenu.Button,
	_label: null,
	_popup: null,
	_started: null,
	_liled: null,
	_loginTimeout: null,
	_folderMenu: null,
	_updateFeedTimeout: null,
	_firstLogin: true,
	_usernameCb: null,
	_passwordCb: null,
	_urlCb: null,

	_init: function() {
		this.parent(0.0,"Liled",false);

		this._label = new St.Label();
		this._label.set_text('Liled');
		this.actor.add_actor(this._label);

		this._liled = new Liled.Liled( );
		this._doWork();

		this._usernameCb = settings.connect("changed::username", Lang.bind(this, this._needLogin ) );
		this._passwordCb = settings.connect("changed::password", Lang.bind(this, this._needLogin ) );
		this._urlCb = settings.connect("changed::url", Lang.bind(this, this._needLogin ) );
	},
	_login: function( success, message ) {
		if( !success ) {
			this.menu.removeAll();
			if( message )
				Main.notifyError("Liled!", message);
			return;
		}

		if( this._firstLogin)
			this._firstLogin = false;
		else
			Main.notify("Liled", "login!");

		this._doWork(); /* Loging OK, we need to update feeds now*/
	},
	_doWork: function() {
		if( this._updateFeedTimeout != null ) 
			Mainloop.source_remove( this._updateFeedTimeout );

		if( this.menu.isOpen ) { 
			/*don't update menu if it is opened*/
			this._updateFeedTimeout = Mainloop.timeout_add(10000, Lang.bind(this, this._doWork) );
			return;
		}

		if( this._liled.user == null ) { 
			this.updateLogin();
		}else { 
			this._liled.updateFolders( Lang.bind(this, this.updateFeeds) );
			this._liled.updateFeeds( Lang.bind(this, this.updateFeeds) );
		}

		if( this._updateFeedTimeout != null ) 
			Mainloop.source_remove( this._updateFeedTimeout );
		this._updateFeedTimeout = Mainloop.timeout_add( this._getUpdateTimeout(), Lang.bind(this, this._doWork) );
	},
	_openFeed: function( item ) {
		let appSys = Shell.AppSystem.get_default();
		let app = appSys.lookup_app('firefox.desktop');
		if( app == null )
			app = appSys.lookup_app('chromium.desktop');
		try { 
			
			let baseURL = settings.get_string('url');
			let endWith = "/app/index.php";
			if( baseURL.indexOf(endWith, baseURL.length - endWith.length) !== -1 )
				baseURL = baseURL.substring(0,  baseURL.length - endWith.length );
			
			app.launch(0, [baseURL + '/index.php?action=selectedFeed&feed=' + item.feed.id], -1, null  );

		}catch( e ){
			reportFailure(e);
		}
	},
	_needLogin: function() {
		if( this._loginTimeout != null ) 
			Mainloop.source_remove( this._loginTimeout );

		this._loginTimeout = Mainloop.timeout_add(3000, Lang.bind(this, this.updateLogin) );
	},
	updateLogin: function() {
		if( this._loginTimeout != null )
		{
			Mainloop.source_remove( this._loginTimeout );
			this._loginTimeout = null;
		}

		this._liled.login(  settings.get_string('url'), 
				settings.get_string('username'), 
				settings.get_string('password'), 
				Lang.bind(this, this._login));
	},
	updateFeeds: function ( success, message ) {
		if( !success )
			return;

		let feeds = this._liled.feeds;
		let folders = this._liled.folders;
		if ( feeds === undefined || folders === undefined )
			return;

		this.menu.removeAll();
		try{
			for( let i = 0 ; i < folders.length ; i++ )
			{
				let folder = folders[i];
				feeds = this._liled.getFeeds( folder.id );
				let sub = new PopupMenu.PopupSubMenuMenuItem(folder.name);
				for( let j = 0 ; j < feeds.length ; j++ )
				{
					let feed = feeds[j];
					let item = new PopupMenu.PopupMenuItem( feed.name + " (" + feed.notRead + ")" );
					item.feed = feed;
					item.connect("activate", Lang.bind(this, this._openFeed, feed ) );

					sub.menu.addMenuItem( item );
				}
				this.menu.addMenuItem(sub);
			}

		}catch(e){
			reportFailure(e);
		}
	},
	_getUpdateTimeout: function() {
		return settings.get_int('refresh') * 1000;
	},
	destroy: function() {
		if( this._loginTimeout != null ) 
			Mainloop.source_remove( this._loginTimeout );
		if( this._updateFeedTimeout != null ) 
			Mainloop.source_remove( this._updateFeedTimeout );
		if( this._usernameCb )
			settings.disconnect( this._usernameCb );
		if( this._passwordCb )
			settings.disconnect( this._passwordCb );
		if( this._urlCb )
			settings.disconnect( this._urlCb );
		this.parent();
	}
});

function getSettings() {
	let schema_id = "org.gnome.shell.extensions.liled";
	let schema_path = ExtensionUtils.getCurrentExtension().path + "/schemas";
	let schema_source = Gio.SettingsSchemaSource.new_from_directory(schema_path,
			Gio.SettingsSchemaSource.get_default(),
			false);
	if (!schema_source) {
		throw new Error("Local schema directory for " + schema_id + " is missing");
	}
	let schema = schema_source.lookup(schema_id, true);
	if (!schema) {
		throw new Error("Schema " + schema_id + " is missing.  Has glib-compile-schemas been called for it?");
	}
	return new Gio.Settings({settings_schema: schema});
}

function init(metadata) {

}

function enable() {
	try {
		settings = getSettings();
		mainPanel = new MainPanel();

		Main.panel.addToStatusArea('Liled',mainPanel);
	} catch(err) {
		reportFailure(err);
		mainPanel.destroy();
		mainPanel = null;
	}
}

function disable() {
	if(mainPanel) {
		mainPanel.destroy();
		mainPanel = null;
	}
	settings = null;
}
